@@ -27,17 +27,26 @@
 #include <config.h>
 #endif
 
+#ifdef GNOME_GTKHTML_EDITOR_SHLIB
+#include <glib/gi18n-lib.h>
+#else
 #include <glib/gi18n.h>
+#endif
 #include <string.h>
 #include <gnome.h>
 #include <bonobo.h>
 #include <stdio.h>
 #include <glib.h>
+#include <glib/gstdio.h>
 #include <errno.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 
+#ifndef O_BINARY
+#define O_BINARY 0
+#endif
+
 #include <gtk/gtkscrolledwindow.h>
 
 #include <glade/glade.h>
@@ -45,6 +54,7 @@
 #include "Editor.h"
 
 #include "gtkhtml.h"
+#include "gtkhtml-private.h"
 #include "gtkhtml-properties.h"
 #include "htmlcursor.h"
 #include "htmlengine.h"
@@ -162,6 +172,7 @@ release (GtkWidget *widget, GdkEventButton *event, GtkHTMLControlData *cd)
 	HTMLEngine *e = cd->html->engine;
 	GtkHTMLEditPropertyType start = GTK_HTML_EDIT_PROPERTY_BODY;
 	gboolean run_dialog = FALSE;
+	char *filename;
 
 	if (cd->obj) {
 		switch (HTML_OBJECT_TYPE (cd->obj)) {
@@ -175,8 +186,10 @@ release (GtkWidget *widget, GdkEventButton *event, GtkHTMLControlData *cd)
 			;
 		}
 		if (run_dialog) {
+			filename = gnome_icon_theme_lookup_icon (cd->icon_theme, "stock_properties", 16, NULL, NULL);
 			cd->properties_dialog = gtk_html_edit_properties_dialog_new (cd, _("Properties"), 
-										     gnome_icon_theme_lookup_icon (cd->icon_theme, "stock_properties", 16, NULL, NULL));
+										     filename);
+			g_free(filename);
 			html_cursor_jump_to (e->cursor, e, cd->obj, 0);
 			html_engine_disable_selection (e);
 			html_engine_set_mark (e);
@@ -239,12 +252,9 @@ load_from_file (GtkHTML *html,
 	int fd;
         char *path;
 
-        if (strncmp (url, "file:", 5) == 0)
-		path = g_filename_from_uri(url, NULL, NULL);
-	else
-		path = g_strdup(url);
+	path = gtk_html_filename_from_uri (url);
 
-	fd = open (path, O_RDONLY);
+	fd = g_open (path, O_RDONLY|O_BINARY, 0);
 	g_free(path);
 
 	if (fd == -1) {
