@@ -50,14 +50,12 @@ draw_background_helper (HTMLTableCell *cell,
 {
 	HTMLObject *o;
 	HTMLClueV *cluev;
-	HTMLTable  *t;
 	GdkPixbuf  *pixbuf = NULL;
 	GdkColor   *color = NULL;
 	int pixel_size = html_painter_get_pixel_size (p);
 
 	o = HTML_OBJECT (cell);
 	cluev = HTML_CLUEV (cell);
-	t = HTML_IS_TABLE (o->parent) ? HTML_TABLE (o->parent) : NULL;
 
 	if (cell->have_bg) {
 		if (! cell->bg_allocated) {
@@ -137,7 +135,7 @@ calc_min_width (HTMLObject *o,
 		HTMLPainter *painter)
 {
 	if (HTML_TABLE_CELL (o)->no_wrap)
-		return MAX ((* HTML_OBJECT_CLASS (parent_class)->calc_min_width) (o, painter),
+		return MAX ((* HTML_OBJECT_CLASS (parent_class)->calc_preferred_width) (o, painter),
 			    o->flags & HTML_OBJECT_FLAG_FIXEDWIDTH
 			    ? HTML_TABLE_CELL (o)->fixed_width * html_painter_get_pixel_size (painter)
 			    : 0);
