@@ -1,6 +1,8 @@
 #include <string.h>
 #include <stdio.h>
+#include <glib/gstring.h>
 #include <gtk/gtkmain.h>
+#include <gtk/gtkscrolledwindow.h>
 #include <gtk/gtkwindow.h>
 #include "gtkhtml.h"
 #include "htmlclue.h"
@@ -12,6 +14,10 @@
 #include "htmlengine-edit-cut-and-paste.h"
 #include "htmlengine-edit-movement.h"
 #include "htmlengine-edit-text.h"
+#include "htmlengine-save.h"
+#include "htmlselection.h"
+#include "htmltable.h"
+#include "htmltablecell.h"
 #include "htmltext.h"
 
 typedef struct {
@@ -24,24 +30,42 @@ static int test_cursor_beol_rtl (GtkHTML *html);
 static int test_cursor_left_right_on_items_boundaries (GtkHTML *html);
 static int test_cursor_left_right_on_lines_boundaries (GtkHTML *html);
 static int test_cursor_left_right_on_lines_boundaries_rtl (GtkHTML *html);
+static int test_cursor_left_right_on_lines_boundaries_wo_white (GtkHTML *html);
 static int test_cursor_around_containers (GtkHTML *html);
+static int test_cursor_around_image (GtkHTML *html);
+static int test_cursor_around_anchors (GtkHTML *html);
 
 static int test_quotes_in_div_block (GtkHTML *html);
+static int test_quotes_in_table (GtkHTML *html);
 static int test_capitalize_upcase_lowcase_word (GtkHTML *html);
 static int test_delete_nested_cluevs_and_undo (GtkHTML *html);
+static int test_insert_nested_cluevs (GtkHTML *html);
+static int test_indentation_plain_text (GtkHTML *html);
+static int test_indentation_plain_text_rtl (GtkHTML *html);
+static int test_table_cell_parsing (GtkHTML *html);
+static int test_delete_around_table (GtkHTML *html);
 
 static Test tests[] = {
 	{ "cursor movement", NULL },
 	{ "left/right on items boundaries", test_cursor_left_right_on_items_boundaries },
 	{ "left/right on lines boundaries", test_cursor_left_right_on_lines_boundaries },
 	{ "left/right on lines boundaries (RTL)", test_cursor_left_right_on_lines_boundaries_rtl },
+	{ "left/right on line boundaries - without white space", test_cursor_left_right_on_lines_boundaries_wo_white },
 	{ "begin/end of line", test_cursor_beol },
 	{ "begin/end of line (RTL)", test_cursor_beol_rtl },
 	{ "around containers", test_cursor_around_containers },
+	{ "around image", test_cursor_around_image },
+	{ "around anchors", test_cursor_around_anchors },
 	{ "various fixed bugs", NULL },
 	{ "outer quotes inside div block", test_quotes_in_div_block },
+	{ "outer quotes inside table", test_quotes_in_table },
 	{ "capitalize, upcase/lowcase word", test_capitalize_upcase_lowcase_word },
 	{ "delete across nested cluev's and undo", test_delete_nested_cluevs_and_undo },
+	{ "insert nested cluev's", test_insert_nested_cluevs },
+	{ "indentation in plain text", test_indentation_plain_text },
+	{ "indentation in plain text (RTL)", test_indentation_plain_text_rtl },
+	{ "table cell parsing", test_table_cell_parsing },
+	{ "delete around table", test_delete_around_table },
 	{ NULL, NULL }
 };
 
@@ -53,6 +77,30 @@ static void load_editable (GtkHTML *html, char *s)
 	gtk_html_set_editable (html, TRUE);
 }
 
+static gboolean
+plain_save_receiver (gpointer engine, const char *data, size_t len, gpointer user_data)
+{
+	GString *str = (GString *) user_data;
+
+	g_string_append_len (str, data, len);
+
+	return TRUE;
+}
+
+static char *
+get_plain (GtkHTML *html)
+{
+	GString *str = g_string_new (0);
+	char *rv;
+
+	html_engine_save_plain (html->engine, plain_save_receiver, str);
+
+	rv = str->str;
+	g_string_free (str, FALSE);
+
+	return rv;
+}
+
 static int test_delete_nested_cluevs_and_undo (GtkHTML *html)
 {
 	load_editable (html, "<div>abc</div><div>efg</div>");
@@ -85,6 +133,45 @@ static int test_delete_nested_cluevs_and_undo (GtkHTML *html)
 	return TRUE;
 }
 
+static int test_insert_nested_cluevs (GtkHTML *html)
+{
+	load_editable (html, "text");
+
+	gtk_html_insert_html (html, "<div>text in div block</div>");
+
+	return TRUE;
+}
+
+static int test_indentation_plain_text (GtkHTML *html)
+{
+	char *str;
+
+	load_editable (html, "abc<div align=right>abc</div>");
+
+	str = get_plain (html);
+	if (!str || strcmp (str, "abc\n                                                                     abc\n"))
+		return FALSE;
+
+	g_free (str);
+
+	return TRUE;
+}
+
+static int test_indentation_plain_text_rtl (GtkHTML *html)
+{
+	char *str;
+
+	load_editable (html, "שנבגקכעי<div align=left>שנבגקכעי</div>");
+
+	str = get_plain (html);
+	if (!str || strcmp (str, "שנבגקכעי\n                                                                שנבגקכעי\n"))
+		return FALSE;
+
+	g_free (str);
+
+	return TRUE;
+}
+
 static int test_cursor_around_containers (GtkHTML *html)
 {
 	load_editable (html, "abc<table><tr><td>abc</td></tr></table>");
@@ -124,6 +211,104 @@ static int test_cursor_around_containers (GtkHTML *html)
 	return TRUE;
 }
 
+static int test_cursor_around_image (GtkHTML *html)
+{
+	load_editable (html, "<pre>abc <img src=none> abc");
+
+	if (!html_cursor_right (html->engine->cursor, html->engine)
+	    || !html_cursor_right (html->engine->cursor, html->engine)
+	    || !html_cursor_right (html->engine->cursor, html->engine)
+	    || !html_cursor_right (html->engine->cursor, html->engine)
+	    || !html_cursor_right (html->engine->cursor, html->engine)
+	    || html->engine->cursor->offset != 1
+	    || html->engine->cursor->position != 5
+	    || !html_cursor_right (html->engine->cursor, html->engine)
+	    || html->engine->cursor->offset != 1
+	    || html->engine->cursor->position != 6
+	    || !html_cursor_left (html->engine->cursor, html->engine)
+	    || !html_cursor_left (html->engine->cursor, html->engine)
+	    || html->engine->cursor->offset != 4
+	    || html->engine->cursor->position != 4) {
+		fprintf (stderr, "\npos: %d off: %d\n", html->engine->cursor->position, html->engine->cursor->offset);
+		return FALSE;
+	}
+
+	while (html_cursor_left (html->engine->cursor, html->engine))
+		;
+
+	if (html->engine->cursor->position != 0 || html->engine->cursor->offset != 0)
+		return FALSE;
+
+	return TRUE;
+}
+
+static int test_cursor_around_anchors (GtkHTML *html)
+{
+	load_editable (html, "<pre>a<a name=anchor>b");
+
+	html_cursor_jump_to_position (html->engine->cursor, html->engine, 0);
+	if (html->engine->cursor->offset != 0
+	    || html->engine->cursor->position != 0)
+		return FALSE;
+
+	html_cursor_down (html->engine->cursor, html->engine);
+
+	if (html->engine->cursor->offset != 1
+	    || html->engine->cursor->position != 2)
+		return FALSE;
+
+	html_cursor_up (html->engine->cursor, html->engine);
+
+	if (html->engine->cursor->offset != 0
+	    || html->engine->cursor->position != 0)
+		return FALSE;
+
+	html_cursor_end_of_line (html->engine->cursor, html->engine);
+
+	if (html->engine->cursor->offset != 1
+	    || html->engine->cursor->position != 2)
+		return FALSE;
+
+	html_cursor_beginning_of_line (html->engine->cursor, html->engine);
+
+	if (html->engine->cursor->offset != 0
+	    || html->engine->cursor->position != 0)
+		return FALSE;
+
+	load_editable (html, "<pre>a<a name=anchor1><img src=src><a name=anchor2>b");
+
+	html_cursor_jump_to_position (html->engine->cursor, html->engine, 0);
+	if (html->engine->cursor->offset != 0
+	    || html->engine->cursor->position != 0)
+		return FALSE;
+
+	html_cursor_down (html->engine->cursor, html->engine);
+
+	if (html->engine->cursor->offset != 1
+	    || html->engine->cursor->position != 3)
+		return FALSE;
+
+	html_cursor_up (html->engine->cursor, html->engine);
+
+	if (html->engine->cursor->offset != 0
+	    || html->engine->cursor->position != 0)
+		return FALSE;
+
+	html_cursor_end_of_line (html->engine->cursor, html->engine);
+
+	if (html->engine->cursor->offset != 1
+	    || html->engine->cursor->position != 3)
+		return FALSE;
+
+	html_cursor_beginning_of_line (html->engine->cursor, html->engine);
+
+	if (html->engine->cursor->offset != 0
+	    || html->engine->cursor->position != 0)
+		return FALSE;
+
+	return TRUE;
+}
+
 static int test_cursor_left_right_on_items_boundaries (GtkHTML *html)
 {
 	load_editable (html, "ab<b>cde</b>ef");
@@ -240,6 +425,27 @@ static int test_cursor_left_right_on_lines_boundaries_rtl (GtkHTML *html)
 	return TRUE;
 }
 
+static int test_cursor_left_right_on_lines_boundaries_wo_white (GtkHTML *html)
+{
+	load_editable (html, "oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo%next line");
+
+	html_cursor_jump_to_position (html->engine->cursor, html->engine, 81);
+
+	if (html->engine->cursor->offset != 81
+	    || html->engine->cursor->position != 81
+	    || !html_cursor_left (html->engine->cursor, html->engine)
+	    || html->engine->cursor->offset != 80
+	    || html->engine->cursor->position != 80
+	    || !html_cursor_right (html->engine->cursor, html->engine)
+	    || html->engine->cursor->offset != 81
+	    || html->engine->cursor->position != 81)
+		return FALSE;
+
+	printf ("test_cursor_left_right_on_lines_boundaries_wo_white: passed\n");
+
+	return TRUE;
+}
+
 static int test_cursor_beol (GtkHTML *html)
 {
 	load_editable (html, "<pre>simple line\nsecond line\n");
@@ -346,6 +552,42 @@ test_quotes_in_div_block (GtkHTML *html)
 	return TRUE;
 }
 
+static int
+test_quotes_in_table (GtkHTML *html)
+{
+	GByteArray *flow_levels;
+	HTMLEngine *e = html->engine;
+	HTMLTable *table;
+
+	load_editable (html,
+		       "<blockquote type=cite>"
+		       "quoted text"
+		       "<table><tr><td>"
+		       "quoted text without inner cite"
+		       "</td></tr></table>"
+		       "quoted text"
+		       "</blockquote>");
+
+	/* test for right object hierarhy */
+	if (!e->clue || !HTML_CLUE (e->clue)->head || !HTML_CLUE (e->clue)->head->next || !HTML_IS_CLUEFLOW (HTML_CLUE (e->clue)->head->next)
+	    || !HTML_CLUE (HTML_CLUE (e->clue)->head->next)->head || !HTML_IS_TABLE (HTML_CLUE (HTML_CLUE (e->clue)->head->next)->head))
+		return FALSE;
+
+ 	table = HTML_TABLE (HTML_CLUE (HTML_CLUE (e->clue)->head->next)->head);
+
+ 	if (table->totalCols != 1 || table->totalRows != 1 || !table->cells [0][0] || !HTML_IS_TABLE_CELL (table->cells [0][0])
+	    || !HTML_CLUE (table->cells [0][0])->head || !HTML_IS_CLUEFLOW (HTML_CLUE (table->cells [0][0])->head))
+ 		return FALSE;
+
+	flow_levels = HTML_CLUEFLOW (HTML_CLUE (table->cells [0][0])->head)->levels;
+
+	/* test if levels are OK */
+	if (!flow_levels || !flow_levels->len == 0)
+		return FALSE;
+
+	return TRUE;
+}
+
 static int
 test_capitalize_upcase_lowcase_word (GtkHTML *html)
 {
@@ -409,9 +651,85 @@ test_capitalize_upcase_lowcase_word (GtkHTML *html)
 	return TRUE;
 }
 
+static int
+test_delete_around_table (GtkHTML *html)
+{
+	load_editable (html, "<table><tr><td></td></tr></table><br>abc");
+
+	html_cursor_jump_to_position (html->engine->cursor, html->engine, 3);
+
+	if (html->engine->cursor->offset != 0
+	    || html->engine->cursor->position != 3)
+		return FALSE;
+
+	gtk_html_command (html, "delete-back");
+
+	if (html->engine->cursor->offset != 1
+	    || html->engine->cursor->position != 2)
+		return FALSE;
+
+	html_engine_end_of_document (html->engine);
+
+	if (html->engine->cursor->offset != 3
+	    || html->engine->cursor->position != 7)
+		return FALSE;
+
+	return TRUE;
+}
+
+static int
+test_table_cell_parsing (GtkHTML *html)
+{
+	load_editable (html, "<table><tr><td></td></tr></table>");
+
+	html_cursor_jump_to_position (html->engine->cursor, html->engine, 1);
+
+	if (html->engine->cursor->offset != 0
+	    || html->engine->cursor->position != 1)
+		return FALSE;
+
+	/* test that there's flow with text created in the cell and that it's only one flow */
+	if (!HTML_IS_TEXT (html->engine->cursor->object) || !html->engine->cursor->object->parent ||
+	    !HTML_IS_CLUEFLOW (html->engine->cursor->object->parent) || html->engine->cursor->object->parent->next ||
+	    !html->engine->cursor->object->parent->parent || !HTML_IS_TABLE_CELL (html->engine->cursor->object->parent->parent))
+		return FALSE;
+
+	load_editable (html, "<table><tr><td><br></td></tr></table>");
+
+	html_cursor_jump_to_position (html->engine->cursor, html->engine, 1);
+
+	if (html->engine->cursor->offset != 0
+	    || html->engine->cursor->position != 1)
+		return FALSE;
+
+	/* test that there are two flows created in the cell and that they are both containing text */
+	if (!HTML_IS_TEXT (html->engine->cursor->object) || !html->engine->cursor->object->parent ||
+	    !HTML_IS_CLUEFLOW (html->engine->cursor->object->parent) || !html->engine->cursor->object->parent->next ||
+	    !html->engine->cursor->object->parent->parent || !HTML_IS_TABLE_CELL (html->engine->cursor->object->parent->parent) ||
+	    !HTML_IS_CLUEFLOW (html->engine->cursor->object->parent->next) || !HTML_CLUE (html->engine->cursor->object->parent->next)->head ||
+	    !HTML_IS_TEXT (HTML_CLUE (html->engine->cursor->object->parent->next)->head))
+		return FALSE;
+
+	load_editable (html, "<table><tr><td>abc</td></tr></table>");
+
+	html_cursor_jump_to_position (html->engine->cursor, html->engine, 1);
+
+	if (html->engine->cursor->offset != 0
+	    || html->engine->cursor->position != 1)
+		return FALSE;
+
+	/* test that there's flow with text created in the cell and that it's only one flow */
+	if (!HTML_IS_TEXT (html->engine->cursor->object) || !html->engine->cursor->object->parent ||
+	    !HTML_IS_CLUEFLOW (html->engine->cursor->object->parent) || html->engine->cursor->object->parent->next ||
+	    !html->engine->cursor->object->parent->parent || !HTML_IS_TABLE_CELL (html->engine->cursor->object->parent->parent))
+		return FALSE;
+
+	return TRUE;
+}
+
 int main (int argc, char *argv[])
 {
-	GtkWidget *win, *html_widget;
+	GtkWidget *win, *sw, *html_widget;
 	GtkHTML *html;
 	int i = 0, n_all, n_successful;
 
@@ -422,7 +740,13 @@ int main (int argc, char *argv[])
 	gtk_html_load_empty (html);
 	gtk_html_set_editable (html, TRUE);
 	win = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-	gtk_container_add (GTK_CONTAINER (win), html_widget);
+	sw = gtk_scrolled_window_new (NULL, NULL);
+	gtk_widget_set_size_request (win, 600, 400);
+	gtk_container_add (GTK_CONTAINER (sw), html_widget);
+	gtk_container_add (GTK_CONTAINER (win), sw);
+
+/* 	gtk_widget_show_all (win); */
+/* 	gtk_widget_show_now (win); */
 
 	n_all = n_successful = 0;
 
